% example.m: An example to demonstrate spectral sparsification of
% graphs.
%
%   Runs a simple example of sparsifying a graph. We construct a graph
%   consisting of 40 nodes located along a circle of unit radius. We
%   construct the adjacency matrix using inverse of distance between 
%   the nodes as the edge weights. Here, the diagonal entries of the 
%   matrix are set to zero ( no self - loops). We then plot the graph 
%   structure and sparsity of the (original graph) adjacency matrix.
%
%   Choosing an approximation order of epsilon = 1, we perform spectral
%   sparsification of the graph. We then plot the graph 
%   structure and sparsity of the (sparse graph) adjacency matrix.
%
%   For more details, see
%
%       SPIELMAN, D. A. & SRIVASTAVA, N. 2011 Graph sparsification by 
%       effective resistances. SIAM J. Sci. Comput. 40 (6), 1913-1926.
%
%       NAIR, A. G. & TAIRA, K. 2015 Network-theoretic approach to 
%       sparsified discrete vortex dynamics. J. Fluid Mech. 768, 549-571.
%
% Requires: spectral_sparsify
%           
% Tested using Matlab 8.1.0.604 (R2013a) for Mac OSX.
%
% Authors:
%   Aditya Nair
%   Kunihiko Taira
%
% Created:
%   December 2016.


function example()

%% Initialize parameters

n     = 40;                              % number of nodes

% location of the nodes

theta = linspace(0,2*pi-2*pi/(n-1),n)';  % angular cordinate
x     = cos(theta);                      % x-cordinate
y     = sin(theta);                      % y-cordinate

%% Original graph

% Adjacency matrix (original)

A = zeros(n,n);                          % initialize adjacency matrix
for i = 1:length(A)
    for j = 1:length(A)
        if i~=j                          % Avoiding self-loops
            % distance as measure
            A(i,j) = 1/sqrt((x(i)-x(j))^2+(y(i)-y(j))^2);
        end
    end
end
maxEwt = max(A(:));                      % maximum edge weight 

% Original graph plotting

[Xo,Yo] = gplot(A,[x y]);                % Using gplot
figure;
subplot(221);              
plot(Xo,Yo,'o-','Linewidth',0.001,'Color',[0.4 0.4 0.4],...
    'MarkerFaceColor','k');              % plot nodes and edges
axis('equal');
xlim([-1.1 1.1]);ylim([-1.1 1.1]);       % set axis limits
axis off;
title('Original graph (\epsilon = 0)','Fontsize',10,'Fontname','Times');
set(gca,'Fontsize',10,'Fontname','Times');
print('-depsc','org_graph.eps');          
close;

% Sparsity plot of original graph (weighted spy plot)

figure;
sparsity_plot(A,maxEwt);
title('$\bf{A}_{\mathcal{G}}$','Interpreter','Latex');
print('-depsc','org_graph_sparsity.eps');  
close;

%% Sparsified graph

% Spectral sparsification

epsilon = 1;                             % Approximation order (range: 0-1)
As = spectral_sparsify(A,epsilon);       % spectral sparsification

% Sparse graph plotting

[Xs,Ys] = gplot(As,[x y]);               % Using gplot
figure;
subplot(221);
plot(Xs,Ys,'o-','Linewidth',0.001,'Color',[0.4 0.4 0.4],...
    'MarkerFaceColor','k');              % plot nodes and edges
axis('equal');
xlim([-1.1 1.1]);ylim([-1.1 1.1]);       % set axis limits
axis off;
title('Sparsified graph (\epsilon = 1)','Fontsize',10,'Fontname','Times');
set(gca,'Fontsize',10,'Fontname','Times');
print('-depsc','sparse_graph.eps');      
close;

% Sparsity plot of sparse graph (weighted spy plot)

figure;
sparsity_plot(As,maxEwt);
title('$\bf{A}_{\mathcal{G}_S}$','Interpreter','Latex');
print('-depsc','sparse_graph_sparsity.eps'); 
close;

% Comparing eigenspectra of original and sparse graph
figure;
subplot(221);
plot(eig(A),'k.','Markersize',10);hold on;
plot(eig(As),'ro','Markersize',6);hold on;
xlim([0 50]);ylim([-60 60]);
box off;
legend('Original','Sparse',2);
legend boxoff;
xlabel('i','Fontsize',10,'Fontname','Times');ylabel('\lambda','Fontsize',10,'Fontname','Times');
title('Eigenspectra comparison','Fontsize',10,'Fontname','Times');
set(gca,'Fontsize',10,'Fontname','Times');
print('-depsc','spectra_comp.eps'); 
close;

end

function sparsity_plot(S,maxEwt)
[m, n] = size(S);
[~, ~] = meshgrid(1:m, 1:n);
nonzeroInd = find(S);
[x, y] = ind2sub([m n], nonzeroInd);
subplot(221);
patch(x, y, S(nonzeroInd), ...
    'Marker', 's', 'MarkerFaceColor', 'flat', 'MarkerSize', 4, ...
    'EdgeColor', 'none', 'FaceColor', 'none');
set(gca, 'XLim', [1, n], 'YLim', [1, m],'YDir', 'reverse', ...
    'PlotBoxAspectRatio', [n, m, 1]);
colormap();
caxis([eps, maxEwt/2]);
axis tight;
box on;
hcb = colorbar;
set(gca,'Fontsize',10,'Fontname','Times');
set(hcb,'Fontsize',10,'Fontname','Times');
end


