% spectral_sparsify  performs spectral sparsification of graphs
% 
% Usage:
%   
%   As = spectral_sparsify(A,epsilon)
%
%   Input
%   A       : Adjacency matrix (weighted/undirected)
%   epsilon : approximation order with the range from 0 to 1
%             epsilon = 0  : original graph
%             epsilon = 0.5: moderately sparsified graph
%             epsilon = 1  : heavily sparsified graph
%
%   Output
%   As      : sparified adjacency matrix (weighted/undirected)
%
%
%   For details, see
%
%       SPIELMAN, D. A. & SRIVASTAVA, N. 2011 Graph sparsification by 
%       effective resistances. SIAM J. Sci. Comput. 40 (6), 1913-1926.
%
%       NAIR, A. G. & TAIRA, K. 2015 Network-theoretic approach to 
%       sparsified discrete vortex dynamics. J. Fluid Mech. 768, 549-571.
%
% Tested using Matlab 8.1.0.604 (R2013a) for Mac OSX.
%
% Authors:
%   Aditya G. Nair
%   Kunihiko Taira
%
% Created:
%   December 2016.


function As = spectral_sparsify(A,epsilon)

%% Initializing Parameters

N        = length(A);                     % Number of vortices
A        = (A + A')/2;                    % Make adjacency matrix symmetric 
L        = diag(sum(A))-A;                % Laplacian Matrix
L_plus   = pinv(L);                       % Moore-Penrose Pseudoinverse
edges1   = find(triu(A>0));               % Indices of all edges

%% Effective resistance computation

[pp,qq]= ind2sub([N,N],edges1);           % Index numbers of edges
Re     = zeros(length(edges1),1);         % Effective resistance
we     = zeros(length(edges1),1);         % Weights of original graph

for i=1:length(pp)
    we(i) = A(pp(i),qq(i));               % Edge weights
    Re(i) = (L_plus(pp(i),pp(i)) - L_plus(qq(i),pp(i)))-...
          (L_plus(pp(i),qq(i)) - L_plus(qq(i),qq(i)));%effective resistance 
end

pe = we.*Re;                         % Edge probability 
pe = pe/sum(pe);                     % Normalizing probabilities 
edges = [0;cumsum(pe)];              % Creating cumulative probability bins

%% Random sampling procedure

if (epsilon>0) && (epsilon<=1)
    q    = ceil(8*N*log2(N)/(epsilon^2));% Number of samples
elseif epsilon == 0
    As = A;                              % return original graph
    return;
else
    msg = 'Please enter approximation order in range of 0 to 1';
    error(msg);
end
el_G = zeros(length(edges1),1);      % Sparsified edge list
r = rand(q,1);                       % Choose random values between 0 and 1
[num,~] = histc(r,edges);            % Get bin index of the random number

%% Create sparse edges  (el_G: sparsified edgelist)

el_G(1:length(edges)-1,1) = el_G(1:length(edges)-1,1) ...
+num(1:length(edges)-1).*(we(1:length(edges)-1)./(pe(1:length(edges)-1)*q));

%% Create sparsified adjacency matrix

As = edgeL2adj(el_G,pp,qq,N);     % convert edgelist to adjacency mat
As = triu(As,-1)'+As; % symmetric adjacency matrix

%% Edge list to adjacency matrix

function adj=edgeL2adj(el,pp,qq,n)
adj = zeros(n,n);
for i=1:size(el,1); 
    adj(pp(i),qq(i))=el(i,1);
end


